using System;
using System.IO;
using System.Text;
using NUnit.Framework;

namespace Smdn.Formats.Mime.Formatting {
  [TestFixture]
  public class Base64ContentEncodingStreamTest {
    [Test]
    public void TestWrite_ExactlyBlockSize()
    {
      using (var outputStream = new MemoryStream()) {
        using (var base64stream = new Base64ContentEncodingStream(outputStream, MimeFormat.Standard)) {
          base64stream.Write(new byte[] { 0x41, 0x42, 0x43 }, 0, 3); // 'A' 'B' 'C'
        }

        Assert.AreEqual("QUJD", Encoding.ASCII.GetString(outputStream.ToArray()));
      }
    }

    [Test]
    public void TestWrite_GreaterThanBlockSize()
    {
      using (var outputStream = new MemoryStream()) {
        using (var base64stream = new Base64ContentEncodingStream(outputStream, MimeFormat.Standard)) {
          base64stream.Write(new byte[] { 0x41, 0x42, 0x43, 0x44 }, 0, 4); // 'A' 'B' 'C' 'D'
        }

        Assert.AreEqual("QUJDRA==", Encoding.ASCII.GetString(outputStream.ToArray()));
      }
    }

    [Test]
    public void TestWriteByte_ExactlyBlockSize()
    {
      using (var outputStream = new MemoryStream()) {
        using (var base64stream = new Base64ContentEncodingStream(outputStream, MimeFormat.Standard)) {
          base64stream.WriteByte(0x41); // 'A'
          base64stream.WriteByte(0x42); // 'B'
          base64stream.WriteByte(0x43); // 'C'
        }

        Assert.AreEqual("QUJD", Encoding.ASCII.GetString(outputStream.ToArray()));
      }
    }

    [Test]
    public void TestWriteByte_GreaterThanBlockSize()
    {
      using (var outputStream = new MemoryStream()) {
        using (var base64stream = new Base64ContentEncodingStream(outputStream, MimeFormat.Standard)) {
          base64stream.WriteByte(0x41); // 'A'
          base64stream.WriteByte(0x42); // 'B'
          base64stream.WriteByte(0x43); // 'C'
          base64stream.WriteByte(0x44); // 'D'
        }

        Assert.AreEqual("QUJDRA==", Encoding.ASCII.GetString(outputStream.ToArray()));
      }
    }
  }
}